import 'package:flutter/material.dart';
import 'package:json_annotation/json_annotation.dart';

class StoryType {
  static String get image => "image";
  static String get video => "video";
  static String get quiz => "quiz";
  static String get quizList => "quizList";
}

@JsonSerializable(explicitToJson: true)
class Story {
  final String id;
  final String title;
  final String thumbnail;
  final StoryUser user;
  final DateTime recordedAt;
  final String color;
  final String textColor;
  final StoryBackground background;
  final StoryData data;

  Story(
    this.id,
    this.title,
    this.thumbnail,
    this.user,
    this.recordedAt,
    this.color,
    this.textColor,
    this.background,
    this.data,
  );

  Story.named({
    this.id,
    this.title,
    this.thumbnail,
    this.user,
    this.recordedAt,
    this.color,
    this.textColor,
    this.background,
    this.data,
  });

  Story copyWith({String id, String title, data, recordedAt, thumbnail, user}) {
    return Story.named(
        id: id ?? this.id,
        title: title ?? this.title,
        data: data ?? this.data,
        background: background ?? this.background,
        recordedAt: recordedAt ?? this.recordedAt,
        thumbnail: thumbnail ?? this.thumbnail,
        user: user ?? this.user,
        color: color ?? this.color,
        textColor: textColor ?? this.textColor);
  }

  Color get materialColor => Color(int.tryParse(color) ?? 0xffeeeeee);

  Color get materialTextColor => Color(int.tryParse(textColor) ?? 0xffeeeeee);
}

@JsonSerializable(explicitToJson: true)
class StoryUser {
  final String id;
  final String name;
  final String profilePic;

  StoryUser(this.id, this.name, this.profilePic);
  StoryUser.named({this.id, this.name, this.profilePic});
}

@JsonSerializable(explicitToJson: true)
abstract class StoryBackground {
  final String type;

  StoryBackground(
    this.type,
  );
}

@JsonSerializable(explicitToJson: true)
class StoryVideo extends StoryBackground {
  final String url;
  final String thumbnail;

  StoryVideo(this.url, this.thumbnail) : super(StoryType.video);

  StoryVideo.named({this.url, this.thumbnail}) : super(StoryType.video);
}

@JsonSerializable(explicitToJson: true)
class StoryImage extends StoryBackground {
  final String url;

  StoryImage(this.url) : super(StoryType.image);

  StoryImage.named({this.url}) : super(StoryType.image);
}

@JsonSerializable(explicitToJson: true)
abstract class StoryData {
  final String type;

  StoryData(
    this.type,
  );
}

@JsonSerializable(explicitToJson: true)
class StoryQuiz extends StoryData {
  final String id;
  final String storyId;
  final String question;
  final List<String> answers;
  final int selectedAnswer;

  StoryQuiz(
      this.id, this.storyId, this.question, this.answers, this.selectedAnswer)
      : super(StoryType.quiz);

  StoryQuiz.named(
      {this.id, this.storyId, this.question, this.answers, this.selectedAnswer})
      : super(StoryType.quiz);

  static StoryQuiz get empty => StoryQuiz.named(
      answers: [null, null, null], id: null, question: null, storyId: null);

  StoryQuiz copyWith(
      {String id,
      String storyId,
      String question,
      List<String> answers,
      int selectedAnswer}) {
    return StoryQuiz.named(
        id: id ?? this.id,
        storyId: storyId ?? this.storyId,
        answers: answers ?? this.answers,
        question: question ?? this.question,
        selectedAnswer: selectedAnswer ?? this.selectedAnswer);
  }
}

class StoryQuizList extends StoryData {
  final List<StoryQuiz> quizzes;
  StoryQuizList(this.quizzes) : super(StoryType.quizList);
}
